/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2001, 2024 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* The NX library and any derived work however based on this software     */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for further options.     */
/*                                                                        */
/* NX and NoMachine are trademarks of NoMachine S.a.r.l.                  */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#include "Log.h"
#include "Error.h"

#define ERROR
#define WARNING
#undef  TEST
#undef  DEBUG

//
// Determine if it is possible to optimize
// the data manipulation functions.
//
/*
FIXME: These functions should be rewritten to
       become members of the Data class.
*/
static SYSTEM Int hostBigEndian  = 0;
static SYSTEM Int storeBigEndian = 0;

void SetHostBigEndian(Int flag)
{
  hostBigEndian = flag;
}

Int HostBigEndian()
{
  return hostBigEndian;
}

Int StoreBigEndian()
{
  return storeBigEndian;
}

//
// Data manipulation functions. Optimizations
// don't work on SPARCs if the buffer is not
// aligned to the word boundary. We should
// check the CPU, not the OS.
//
/*
FIXME: These functions should be rewritten to
       become members of the Data class.
*/
unsigned Int GetUINT(unsigned const Char *buffer, Int bigEndian)
{
  #ifndef __sun

  if (hostBigEndian == bigEndian)
  {
    return *((unsigned short *) buffer);
  }

  #else

  if (hostBigEndian == bigEndian && ((unsigned int) buffer) & 0x1 == 0)
  {
    return *((unsigned short *) buffer);
  }

  #endif

  unsigned Int result;

  if (bigEndian)
  {
    result = *buffer;

    result <<= 8;

    result += buffer[1];
  }
  else
  {
    result = buffer[1];

    result <<= 8;

    result += *buffer;
  }

  return result;
}

unsigned Int GetULONG(unsigned const Char *buffer, Int bigEndian)
{
  //
  // It doesn't work on SPARCs if the buffer
  // is not aligned to word the boundary.
  //

  #ifndef __sun

  if (hostBigEndian == bigEndian)
  {
    return *((unsigned Int *) buffer);
  }

  #else

  if (hostBigEndian == bigEndian && ((unsigned int) buffer) & 0x3 == 0)
  {
    return *((unsigned Int *) buffer);
  }

  #endif

  const unsigned Char *next = (bigEndian ? buffer : buffer + 3);

  unsigned Int result = 0;

  for (Int i = 0; i < 4; i++)
  {
    result <<= 8;

    result += *next;

    if (bigEndian)
    {
      next++;
    }
    else
    {
      next--;
    }
  }

  return result;
}

void PutUINT(unsigned Int value, unsigned Char *buffer, Int bigEndian)
{
  if (hostBigEndian == bigEndian)
  {
    *((unsigned short *) buffer) = value;

    return;
  }

  if (bigEndian)
  {
    buffer[1] = (unsigned char) (value & 0xff);

    value >>= 8;

    *buffer = (unsigned char) value;
  }
  else
  {
    *buffer = (unsigned char) (value & 0xff);

    value >>= 8;

    buffer[1] = (unsigned char) value;
  }
}

void PutULONG(unsigned Int value, unsigned Char *buffer, Int bigEndian)
{
  if (hostBigEndian == bigEndian)
  {
    *((unsigned Int *) buffer) = value;

    return;
  }

  if (bigEndian)
  {
    buffer += 3;

    for (Int i = 4; i > 0; i--)
    {
      *buffer-- = (unsigned char) (value & 0xff);

      value >>= 8;
    }
  }
  else
  {
    for (Int i = 4; i > 0; i--)
    {
      *buffer++ = (unsigned char) (value & 0xff);

      value >>= 8;
    }
  }
}
