/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2001, 2024 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* The NX library and any derived work however based on this software     */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for further options.     */
/*                                                                        */
/* NX and NoMachine are trademarks of NoMachine S.a.r.l.                  */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#ifndef Data_H
#define Data_H

//
// Host endianess utility functions.
//

void SetHostBigEndian(Int flag);

Int HostBigEndian();
Int StoreBigEndian();

//
// Manipulate data in memory according
// to the specified host endianess.
//

unsigned Int GetUINT(unsigned const Char *buffer, Int bigEndian = 0);
unsigned Int GetULONG(unsigned const Char *buffer, Int bigEndian = 0);

void PutUINT(unsigned Int value, unsigned Char *buffer, Int bigEndian = 0);
void PutULONG(unsigned Int value, unsigned Char *buffer, Int bigEndian = 0);

inline unsigned Int GetUINT(const Char *buffer, Int bigEndian = 0)
{
  return GetUINT((unsigned const Char *) buffer, bigEndian);
}

inline unsigned Int GetULONG(const Char *buffer, Int bigEndian = 0)
{
  return GetULONG((unsigned const Char *) buffer, bigEndian);
}

//
// Round the number to the specified
// base.
//

inline unsigned Int RoundUp2(unsigned Int value)
{
  return (value & 0x01 ? value + 1 : value);
}

inline unsigned Int RoundUp4(unsigned Int value)
{
  return (value & 0x03 ? ((value >> 2) + 1) << 2 : value);
}

inline unsigned Int RoundUp8(unsigned Int value)
{
  return (value & 0x07 ? ((value >> 3) + 1) << 3 : value);
}

inline unsigned Int RoundUp16(unsigned Int value)
{
  return (value & 0x0f ? ((value >> 4) + 1) << 4 : value);
}

inline unsigned Int RoundUp32(unsigned Int value)
{
  return (value & 0x1f ? ((value >> 5) + 1) << 5 : value);
}

inline void *AlignUp16(void *value)
{
  return (void *) (((Long Long) value + 15) & ~0x0f);
}

inline void *AlignUp32(void *value)
{
  return (void *) (((Long Long) value + 31) & ~0x1f);
}

inline unsigned Long Long RoundUp2(unsigned Long Long value)
{
  return (value & 0x01 ? value + 1 : value);
}

inline unsigned Long Long RoundUp4(unsigned Long Long value)
{
  return (value & 0x03 ? ((value >> 2) + 1) << 2 : value);
}

inline unsigned Long Long RoundUp8(unsigned Long Long value)
{
  return (value & 0x07 ? ((value >> 3) + 1) << 3 : value);
}

inline unsigned Long Long RoundUp16(unsigned Long Long value)
{
  return (value & 0x0f ? ((value >> 4) + 1) << 4 : value);
}

inline unsigned Long Long RoundUp32(unsigned Long Long value)
{
  return (value & 0x1f ? ((value >> 5) + 1) << 5 : value);
}

inline Int IsOdd(unsigned Int value)
{
  return (value & 0x1);
}

inline Int IsEven(unsigned Int value)
{
  return !(value & 0x1);
}

inline Int IsOdd(Int value)
{
  return (value & 0x1);
}

inline Int IsEven(Int value)
{
  return !(value & 0x1);
}

inline Int GetAbs(Int value)
{
  return (value < 0 ? -value : value);
}

inline Long Long GetAbs(Long Long value)
{
  return (value < 0 ? -value : value);
}

#endif /* Data_H */
