/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2001, 2024 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* The NX library and any derived work however based on this software     */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for further options.     */
/*                                                                        */
/* NX and NoMachine are trademarks of NoMachine S.a.r.l.                  */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#include "Log.h"
#include "Error.h"
#include "List.h"

#define ERROR
#define WARNING
#undef  TEST
#undef  DEBUG

void *List::getValue(Int index) const
{
  if (index < 0 || index >= (Int) size())
  {
    return NULL;
  }
  else if (index == 0)
  {
    return *begin();
  }
  else if (index == (Int) size() - 1)
  {
    return *(--end());
  }
  else
  {
    Int i = 0;

    ConstIterator j = begin();

    while (i++ < index) j++;

    return *j;
  }
}

void List::moveValue(Iterator value, Iterator iterator)
{
  //
  // Optimize the case where we are
  // moving the second element to
  // the head.
  //

  if (value == ++begin() && iterator == begin())
  {
    void *temporary = *value;

    *value = *iterator;

    *iterator = temporary;
  }
  else
  {
    splice(iterator, *this, value);
  }
}

void List::removeValue(void *value)
{
  for (Iterator i = begin(); i != end(); i++)
  {
    if (*i == value)
    {
      erase(i);

      return;
    }
  }

  #ifdef ERROR
  Log() << "List: ERROR! Invalid remove operation on "
        << "element not present in the list.\n";
  #endif

  LogError() << "Invalid remove operation on "
             << "element not present in the list.\n";
}

void List::dumpList() const
{
  if (size() > 0)
  {
    Log() << "List: Dumping " << size()
          << " elements in list " << this
          << ":\n";

    Int i = 1;

    for (ConstIterator j = begin(); j != end(); j++)
    {
      Log() << "List: Element " << i++ << " at "
            << *j << ".\n";
    }
  }
  else
  {
    Log() << "List: No elements found in "
          << this << ".\n";
  }
}
