/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2001, 2024 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* The NX library and any derived work however based on this software     */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for further options.     */
/*                                                                        */
/* NX and NoMachine are trademarks of NoMachine S.a.r.l.                  */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#ifndef Log_H
#define Log_H

#include "Logger.h"

//
// Generally available log facilities
// using the static system logger.
//

LogStream &Log();
LogStream &LogSession();
LogStream &LogError();
LogStream &LogWarning();
LogStream &LogInfo();
LogStream &LogUser();

LogStream &LogMore();
LogStream &LogSessionMore();
LogStream &LogErrorMore();
LogStream &LogWarningMore();
LogStream &LogInfoMore();
LogStream &LogUserMore();

void LogFlush();

//
// Internal implementation of the log
// facilities.
//

class Logger;

LogStream &Log(Logger *logger);
LogStream &LogSession(Logger *logger);
LogStream &LogError(Logger *logger);
LogStream &LogWarning(Logger *logger);
LogStream &LogInfo(Logger *logger);
LogStream &LogUser(Logger *logger);
LogStream &LogMore(Logger *logger);

class Object;

LogStream &Log(Object *object);
LogStream &LogSession(Object *object);
LogStream &LogError(Object *object);
LogStream &LogWarning(Object *object);
LogStream &LogInfo(Object *object);
LogStream &LogUser(Object *object);
LogStream &LogMore(Object *object);

//
// Return the default log stream for
// this system based on the buffering
// mode set in the logger.
//

LogStream &LogDefault();

//
// Safe versions not requiring a lock
// or a logger instance.
//

LogStream &LogSafe();
LogStream &LogSafeMore();
LogStream &LogSafeError();

//
// A version including the class name.
//

LogStream &Log(Logger *logger, const Char *name);

//
// Append to the stream.
//

inline LogStream &LogSessionMore(Logger *logger)
{
  return LogUser(logger);
}

inline LogStream &LogErrorMore(Logger *logger)
{
  return LogUser(logger);
}

inline LogStream &LogWarningMore(Logger *logger)
{
  return LogUser(logger);
}

inline LogStream &LogInfoMore(Logger *logger)
{
  return LogUser(logger);
}

inline LogStream &LogUserMore(Logger *logger)
{
  return LogUser(logger);
}

//
// Flush the log output.
//

void LogFlush(Logger *logger);

//
// Make a log header with the process id,
// the thread id and the current time.
//

void LogHeader(Char *header, Int size);

//
// Other macros and utility functions for
// use in the log facilities.
//

#define Ptr(value) VoidCast(value)

#define Str(string) ((string) == NULL ? "nil" : (string))

#define Dtk(string) "\"" << Str(string) << "\""
#define Stk(string) "'" << Str(string) << "'"
#define Sqr(string) "[" << Str(string) << "]"

#define Itk(value)  "'" << IntCast(value) << "'"
#define Ltk(value)  "'" << LongCast(value) << "'"

#define Oct(value) (value == 0 ? "0"  : "") << oct << (value) << dec
#define Hex(value) (value == 0 ? "0x" : "") << hex << (value) << dec

#define Dec() dec

#endif /* Log_H */
