/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2001, 2024 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* The NX library and any derived work however based on this software     */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for further options.     */
/*                                                                        */
/* NX and NoMachine are trademarks of NoMachine S.a.r.l.                  */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#ifndef StringList_H
#define StringList_H

#include "List.h"
#include "String.h"

class StringList : protected VoidList
{
  public:

  typedef VoidList::Iterator Iterator;

  typedef enum
  {
    KeepEmpty,
    SkipEmpty

  } SplitType;

  StringList(Int exclusive = 0);

  ~StringList();

  Int getSize() const
  {
    return VoidList::getSize();
  }

  Iterator getBegin()
  {
    return VoidList::getBegin();
  }

  Iterator getIterator(Int index);

  Iterator getEnd()
  {
    return VoidList::getEnd();
  }

  const Char *getFirst() const
  {
    return (const Char *) VoidList::getFirst();
  }

  const Char *getLast() const
  {
    return (const Char *) VoidList::getLast();
  }

  const Char *getString(Iterator iterator) const
  {
    return (const Char *) (*iterator);
  }

  const Char *getString(Int index) const;

  void addString(const Char *string);

  void insertString(const Char *string, Iterator iterator);

  void removeString(Iterator i);

  void removeString(const Char *string);

  void removeStrings();

  void replaceString(Iterator iterator, const Char *to);

  Iterator findString(const Char *string);

  StringList *findSubString(const Char *substring, StringList *list = NULL);

  StringList *split(Iterator iterator, const Char *sep,
                               SplitType type = KeepEmpty);

  static StringList *split(const Char *string, const Char *sep,
                               SplitType type = KeepEmpty);

  Char *merge(const Char *sep);

  void merge(Char *&buffer, const Char *sep);

  StringList *copyList();

/*
FIXME: Methods below must be reviewed or
       removed.
*/
  void swapString(Iterator i, Iterator j);
  void moveString(Iterator from, Iterator to);

  void sort();

  StringList *sortByItem(Int item, const Char *sep);

  Int insertString(const Char *string);

  Iterator findSubStringIterator(const Char *substring, int *index = NULL);

  protected:

  static StringList *splitAndSkipEmpty(const Char *string, const Char *sep);
  static StringList *splitAndKeepEmpty(const Char *string, const Char *sep);

  private:

  Int exclusive_;
};

#endif /* StringList_H */
