/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2001, 2024 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* The NX library and any derived work however based on this software     */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for further options.     */
/*                                                                        */
/* NX and NoMachine are trademarks of NoMachine S.a.r.l.                  */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#include "Time.h"
#include "String.h"

#define ERROR
#define WARNING
#undef  TEST
#undef  DEBUG

//
// Reuse the thread specific ctime buffer.
//

#ifndef WIN32

static SYSTEM Char time_[32];

#endif

const Char *strTimeval(const Timeval &ts)
{
  #ifdef WIN32

  Char *string = ctime((time_t *) &ts.tv_sec);

  #else

  Char *string = time_;

  ctime_r((time_t *) &ts.tv_sec, string);

  #endif

  string[24] = '\0';

  return string;
}

//
// Return the time of the day plus the
// microseconds. Use the Thread Local
// Storage data to hold the buffer.
//

const Char *strMsTimeval(const Timeval &ts)
{
  #ifdef WIN32

  Char *string = ctime((time_t *) &ts.tv_sec);

  #else

  Char *string = time_;

  ctime_r((time_t *) &ts.tv_sec, string);

  #endif

  Char format[25];

  StringSizePrint(format, 25, "%.8s %07.3f", string + 11,
                  (Double) ts.tv_usec / 1000);

  StringCopy(string, format);

  return string;
}

//
// Implement the missing gettimeofday()
// function on Windows.
//

#ifdef _MSC_VER

#define DELTA_EPOCH_IN_MICROSECS  11644473600000000Ui64

Int gettimeofday(struct timeval *tv, struct timezone *tz)
{
  FILETIME ft;

  unsigned __int64 tmpres = 0;

  static SYSTEM Int tzflag;

  if (NULL != tv)
  {
    GetSystemTimeAsFileTime(&ft);

    tmpres |= ft.dwHighDateTime;
    tmpres <<= 32;
    tmpres |= ft.dwLowDateTime;

    tmpres /= 10;
    tmpres -= DELTA_EPOCH_IN_MICROSECS;

    tv -> tv_sec  = (long) (tmpres / 1000000UL);
    tv -> tv_usec = (long) (tmpres % 1000000UL);
  }

  if (tz != NULL)
  {
    if (!tzflag)
    {
      _tzset();
      tzflag++;
    }

    tz -> tz_minuteswest = _timezone / 60;
    tz -> tz_dsttime     = _daylight;
  }

  return 0;
}

#endif // _MSC_VER
